#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import pathlib
import tarfile

from datetime import datetime
from typing import List

from .interfaces import DeliveryContextIF, Destination, DestinationDecorator


class TarArchiver(DestinationDecorator):
    """
    This decorator creates a local tar archive. Calls to add_file
    are intercepted and instead the file contents are added to the
    tar archive. When close() is called, we finalize the tar file
    and it is placed in the delivery area.
    """

    def __init__(self, context: DeliveryContextIF, underlying: Destination, project_codes: List[str]):
        super().__init__(underlying)
        self.context = context
        self.archive, self.archive_tempfile = self.create_archive(project_codes)

    def create_archive(self, projects):
        tar_date = datetime.now().strftime("%Y%m%d-%H%M%S")
        project_code = "multi_project" if len(projects) > 1 else projects[0]
        archive_filename = f"NRAO_archive_{project_code}_{tar_date}.tar"

        # create the archive as a temporary file
        archive_tempfile = self.underlying.create_file(archive_filename)

        # open the file
        archive = tarfile.open(mode="w:", fileobj=archive_tempfile.file())
        return archive, archive_tempfile

    def add_file(self, file: pathlib.Path, relative_path: str):
        # add the file at the offset
        self.archive.add(file, relative_path)

    def close(self):
        # the tarfile documentation is explicit that closing the archive does NOT close the underlying file
        # therefore, both steps are necessary: the first to finalize the archive,
        # the second to flush and close the file
        self.archive.close()
        self.archive_tempfile.close()

        # now we can proceed
        self.underlying.close()

    def __str__(self):
        return f"tar archive in " + str(self.underlying)
